package com.hikari.regedist.service

import com.hikari.regedist.dao.PolicyItemDao
import com.hikari.regedist.domain.PolicyItemForm
import com.hikari.regedist.entity.PolicyItem
import com.hikari.regedist.support.BusinessError
import com.hikari.regedist.support.BusinessError.DATA_NOT_EXIST
import com.hikari.regedist.support.BusinessError.INSERT_DATA_ALLREADY_EXIST
import gaf3.core.data.PageParam
import gaf3.core.data.PagedData
import gaf3.core.jpa.extension.findAll
import gaf3.core.util.DataBeanHelper
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.validation.annotation.Validated
import javax.validation.Valid

@Service
@Validated
class PolicyItemService {

    @Autowired
    lateinit var policyItemDao: PolicyItemDao

    /**
     * 创建策略项信息
     * @param form 策略项信息表单
     * @return 添加到数据库的策略项信息
     */
    fun create(@Valid form: PolicyItemForm): PolicyItem {
        return PolicyItem().apply {
            check(form)
            DataBeanHelper.Bean2Obj(form, this)
        }.let {
            policyItemDao.save(it)
        }
    }

    /**
     * 修改策略项信息
     * @param id id
     * @param form 策略项信息表单
     * @return 修改后的策略项信息
     */
    fun update(id: String, @Valid form: PolicyItemForm): PolicyItem {
        return policyItemDao.findByIdOrNull(id)?.apply {
            check(form)
            DataBeanHelper.Bean2Obj(form, this)
        }?.also {
            policyItemDao.save(it)
        } ?: throw BusinessError(INSERT_DATA_ALLREADY_EXIST)
    }

    /**
     * 根据条件查询策略项分页列表
     * @param filter 查询条件
     * @param pageParam 分页参数
     * @return 策略项分页列表
     */
    fun find(filter: PolicyItem, pageParam: PageParam): PagedData<PolicyItem> {
        val paged = policyItemDao.findAll(filter, pageParam)
        return PagedData(data = paged.content, total = paged.totalElements.toInt())
    }

    /**
     * 根据id查询策略项信息
     */
    fun findById(id: String): PolicyItem {
        return policyItemDao.findByIdOrNull(id) ?: throw BusinessError(DATA_NOT_EXIST)
    }

    /**
     * 根据id删除策略项
     */
    fun delete(id: String) {
        policyItemDao.findByIdOrNull(id)?.let {
            policyItemDao.delete(it)
        } ?: throw BusinessError(DATA_NOT_EXIST)
    }

    /**
     * 根据策略id和策略键查找策略项信息
     */
    fun findByPolicyIdAndKey(policyId: String, key: String): PolicyItem? {
        return policyItemDao.findByPolicyIdAndItemKey(policyId, key)
    }

    /**
     * 删除指定策略id下所有策略项
     */
    fun deleteByPolicyId(policyId: String) {
        val filter = PolicyItem().apply {
            this.policyId = policyId
        }
        var skip = 0
        val limit = 100
        var total: Int
        do {
            policyItemDao.findAll(filter, PageParam.of(skip, limit)).also {
                skip += limit
                total = it.totalElements.toInt()
            }.content.forEach {
                policyItemDao.delete(it)
            }
        } while (skip < total)
    }

    /**
     * 检查添加或者修改的策略信息是否合理
     * 同一策略id下，策略项名称，策略项键不可相同
     */
    private fun check(form: PolicyItemForm) {
        if (policyItemDao.findByPolicyIdAndName(form.policyId!!, form.name!!) != null) throw BusinessError(INSERT_DATA_ALLREADY_EXIST)
        if (policyItemDao.findByPolicyIdAndItemKey(form.policyId!!, form.itemKey!!) != null) throw BusinessError(INSERT_DATA_ALLREADY_EXIST)
    }

}