package com.hikari.regedist.service

import com.hikari.regedist.PolicyConfiguration
import com.hikari.regedist.dao.PolicyDao
import com.hikari.regedist.domain.PolicyForm
import com.hikari.regedist.domain.PolicyResult
import com.hikari.regedist.entity.Policy
import com.hikari.regedist.entity.PolicyItem
import com.hikari.regedist.support.BusinessError
import com.hikari.regedist.support.BusinessError.*
import gaf3.core.data.PageParam
import gaf3.core.data.PagedData
import gaf3.core.jpa.extension.findAll
import gaf3.core.util.DataBeanHelper
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.util.Assert
import org.springframework.validation.annotation.Validated
import javax.validation.Valid

@Service
@Validated
class PolicyService(val config: PolicyConfiguration) {

    @Autowired
    lateinit var policyDao: PolicyDao

    @Autowired
    lateinit var policyItemService: PolicyItemService

    /**
     * 创建策略信息
     * @param form 策略信息表单
     * @return 添加的策略对象
     */
    fun create(@Valid form: PolicyForm): Policy {
        return Policy().apply {
            DataBeanHelper.Bean2Obj(form, this)
        }.also {
            check(it)
            policyDao.save(it)
        }
    }

    /**
     * 更新策略
     * @param id id
     * @param form 策略信息表单
     */
    fun update(id: String, @Valid form: PolicyForm): Policy {
        return policyDao.findByIdOrNull(id)?.also {
            DataBeanHelper.Bean2Obj(form, it)
            policyDao.save(it)
        } ?: throw BusinessError(DATA_NOT_EXIST)
    }

    /**
     * 查询策略列表
     * @param filter 条件筛选
     * @param pageParam 分页参数
     * @return 分页策略列表信息
     */
    fun find(filter: Policy, pageParam: PageParam): PagedData<Policy> {
        val paged = policyDao.findAll(filter, pageParam)
        return PagedData(data = paged.content, total = paged.totalElements.toInt())
    }


    /**
     * 根据id获取策略对象
     */
    fun findById(id: String): Policy {
        return policyDao.findByIdOrNull(id) ?: throw BusinessError(DATA_NOT_EXIST)
    }

    /**
     * 根据id删除策略对象
     */
    fun delete(id: String) {
        policyDao.findByIdOrNull(id)?.let {
            config.deletePolicy!!.run {
                if (this.toLowerCase() == "hard")
                    deleteAll(it.id)
                else {
                    if (policyDao.existsByParentId(it.id!!)) throw BusinessError(DEL_DATA_HASCHILDREN)
                }
            }
            if (config.cascadeDelPolicyItem) {
                policyItemService.deleteByPolicyId(it.id!!)
            } else {
                throw BusinessError(DEL_DATA_HASCHILDREN, "被删除策略含有策略项，无法删除")
            }
            policyDao.delete(it)
        } ?: throw BusinessError(DATA_NOT_EXIST)
    }

    /**
     * 使用递归删除策略项下所有子项
     */
    fun deleteAll(pid: String?) {
        var skip = 0
        val limit = 100
        do {
            val paged = Policy().apply {
                this.parentId = pid
            }.let {
                find(it, pageParam = PageParam.of(skip, limit))
            }
            paged.data?.forEach {
                deleteAll(it.id)
                if (config.cascadeDelPolicyItem) {
                    policyItemService.deleteByPolicyId(it.id!!)
                } else {
                    throw BusinessError(DEL_DATA_HASCHILDREN, "被删除策略含有策略项，无法删除")
                }
                policyDao.delete(it)
            }
            skip += limit
        } while (skip < paged.total)
    }

    /**
     * 根据策略键和parentId获取策略
     */
    fun findByKeyAndPid(key: String, pid: String?): Policy? {
        Assert.hasText(key, "姓名不可为空")
        return pid?.let {
            policyDao.findByPolicyKeyAndParentId(key, it)
        } ?: policyDao.findByPolicyKeyAndParentIdIsNull(key)
    }

    /**
     * 对添加或修改的策略信息进行校验。
     * 校验规则：同一级别名称，策略键不可相同
     */
    private fun check(policy: Policy) {
        //判断同一级别策略名称、策略键不能相同
        if (policy.parentId == null) {
            if (policyDao.findByNameAndParentIdIsNull(policy.name!!) != null) throw BusinessError(INSERT_DATA_ALLREADY_EXIST, "策略同一级别名称不可相同")
            if (policyDao.findByPolicyKeyAndParentIdIsNull(policy.name!!) != null) throw BusinessError(INSERT_DATA_ALLREADY_EXIST, "策略同一级别键不可相同")
        } else {
            if (policyDao.findByNameAndParentId(policy.name!!, policy.parentId!!) != null) throw BusinessError(INSERT_DATA_ALLREADY_EXIST, "策略同一级别名称不可相同")
            if (policyDao.findByPolicyKeyAndParentId(policy.name!!, policy.parentId!!) != null) throw BusinessError(INSERT_DATA_ALLREADY_EXIST, "策略同一级别键不可相同")
        }
    }

}